@@ -18,7 +18,8 @@
 const int border_width = 32;
 /** 棋子的宽度*/
 /** width of chessman */
-const int chessman_width = 57;
+//const int chessman_width = 57;
+//const int chessman_width = 29;
 
 #define	PLACE_LEFT 0x01
 #define PLACE_RIGHT 0x02
@@ -26,6 +27,7 @@ const int chessman_width = 57;
 #define PIECE_END   48  //棋子结束数字 end number of chessman
 #define PLACE_ALL PLACE_LEFT | PLACE_RIGHT
 #define IMGAGE_DIR DATA_DIR"/wood/"
+#define IMGAGE_SMALL_DIR DATA_DIR"/wood-small/"
 #define HEX_ESCAPE '%'
 
 
@@ -107,13 +109,21 @@ Board::Board(MainWindow& win) :
 	,red_time(500)
 	,black_time(500)
 	,count_time(0)
+	,chessman_width(29)
+	,is_small_board(true)
+	,is_rev_board(false)
+	,m_search_depth(8)
+	,m_usebook(true)
+	,m_human_black(false)
 {
 
 	std::list<Gtk::TargetEntry> listTargets;
 	listTargets.push_back(Gtk::TargetEntry("STRING"));
 	listTargets.push_back(Gtk::TargetEntry("text/plain"));
 
-	this->set_size_request(521,577);
+
+	this->set_size_request(221,277);
+	//this->set_size_request(521,577);
 
 	this->drag_dest_set(listTargets);
 	this->signal_drag_data_received().connect(
@@ -126,6 +136,7 @@ Board::Board(MainWindow& win) :
 	m_engine.init_snapshot(start_fen);
 	m_robot.set_out_slot(sigc::mem_fun(*this, &Board::robot_log));
 	this->set_events(Gdk::BUTTON_PRESS_MASK|Gdk::EXPOSURE_MASK);
+
 	this->show_all();
 
 
@@ -143,29 +154,70 @@ Board::~Board()
 void Board::load_images()
 {
 	bg_image = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"bg.png");
-	this->set_size_request(521,577);
-	chessman_images[BLACK_ADVISOR] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_advisor.png");
-	chessman_images[BLACK_BISHOP] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_bishop.png");
-	chessman_images[BLACK_CANNON] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_cannon.png");
-	chessman_images[BLACK_KING] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_king.png");
-	chessman_images[BLACK_KING_DIE] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_king_die.png");
-	chessman_images[BLACK_KNIGHT] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_knight.png");
-	chessman_images[BLACK_PAWN] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_pawn.png");
-	chessman_images[BLACK_ROOK] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_rook.png");
-	chessman_images[RED_ADVISOR] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_advisor.png");
-	chessman_images[RED_BISHOP] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_bishop.png");
-	chessman_images[RED_CANNON] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_cannon.png");
-	chessman_images[RED_KING] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_king.png");
-	chessman_images[RED_KING_DIE] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_king_die.png");
-	chessman_images[RED_KNIGHT] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_knight.png");
-	chessman_images[RED_PAWN] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_pawn.png");
-	chessman_images[RED_ROOK] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_rook.png");
-	chessman_images[SELECTED_CHESSMAN] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"select.png");
-	chessman_images[NULL_CHESSMAN] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"null.png");
+
+	if(is_small_board){
+		chessman_images[BLACK_ADVISOR] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"black_advisor.png");
+		chessman_images[BLACK_BISHOP] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"black_bishop.png");
+		chessman_images[BLACK_CANNON] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"black_cannon.png");
+		chessman_images[BLACK_KING] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"black_king.png");
+		chessman_images[BLACK_KING_DIE] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"black_king_die.png");
+		chessman_images[BLACK_KNIGHT] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"black_knight.png");
+		chessman_images[BLACK_PAWN] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"black_pawn.png");
+		chessman_images[BLACK_ROOK] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"black_rook.png");
+		chessman_images[RED_ADVISOR] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"red_advisor.png");
+		chessman_images[RED_BISHOP] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"red_bishop.png");
+		chessman_images[RED_CANNON] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"red_cannon.png");
+		chessman_images[RED_KING] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"red_king.png");
+		chessman_images[RED_KING_DIE] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"red_king_die.png");
+		chessman_images[RED_KNIGHT] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"red_knight.png");
+		chessman_images[RED_PAWN] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"red_pawn.png");
+		chessman_images[RED_ROOK] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"red_rook.png");
+		chessman_images[SELECTED_CHESSMAN] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"select.png");
+		chessman_images[NULL_CHESSMAN] = Gdk::Pixbuf::create_from_file(IMGAGE_SMALL_DIR"null.png");
+	}
+	else{
+
+		chessman_images[BLACK_ADVISOR] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_advisor.png");
+		chessman_images[BLACK_BISHOP] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_bishop.png");
+		chessman_images[BLACK_CANNON] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_cannon.png");
+		chessman_images[BLACK_KING] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_king.png");
+		chessman_images[BLACK_KING_DIE] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_king_die.png");
+		chessman_images[BLACK_KNIGHT] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_knight.png");
+		chessman_images[BLACK_PAWN] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_pawn.png");
+		chessman_images[BLACK_ROOK] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"black_rook.png");
+		chessman_images[RED_ADVISOR] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_advisor.png");
+		chessman_images[RED_BISHOP] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_bishop.png");
+		chessman_images[RED_CANNON] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_cannon.png");
+		chessman_images[RED_KING] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_king.png");
+		chessman_images[RED_KING_DIE] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_king_die.png");
+		chessman_images[RED_KNIGHT] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_knight.png");
+		chessman_images[RED_PAWN] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_pawn.png");
+		chessman_images[RED_ROOK] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"red_rook.png");
+		chessman_images[SELECTED_CHESSMAN] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"select.png");
+		chessman_images[NULL_CHESSMAN] = Gdk::Pixbuf::create_from_file(IMGAGE_DIR"null.png");
+
+	}
 	
-
-
 }
+
+void Board::configure_board(int _width)
+{
+	if(is_small_board && _width >=521){
+		is_small_board=false;
+		chessman_width=57;
+		load_images();
+		redraw();
+
+	}
+	if(!is_small_board && _width<521){
+		is_small_board=true;
+		chessman_width=29;
+		load_images();
+		redraw();
+	}
+	
+}
+
 void Board::get_grid_size(int& width, int& height)
 {
 	width = (get_width() - border_width * 2) / 8;
@@ -262,7 +314,7 @@ bool Board::on_button_press_event(GdkEventButton* ev)
 
 			CSound::play(SND_CHOOSE);
 			if (selected_x != -1) {
-				selected_chessman = m_engine.get_piece(selected_x, selected_y);
+				selected_chessman = m_engine.get_piece(selected_x, selected_y,is_rev_board);
 				if(selected_chessman==0){
 					/** 仍然没选中, still not select */
 					selected_chessman = -1;
@@ -271,12 +323,13 @@ bool Board::on_button_press_event(GdkEventButton* ev)
 				}
 				/** 对战状态中，选了黑色棋子无效*/
 				/** choose the black chessman is useless on war */
-				if((is_fight_to_robot()) &&
-					(selected_chessman>31)){
+				if(is_fight_to_robot()){ 
+					if((m_human_black && (selected_chessman <32))||((!m_human_black)&&(selected_chessman>31))){
 						printf("choose black %d\n",selected_chessman);
 						selected_chessman =-1;
 						//draw_select_frame(false); 
 						return true;
+					}
 				}
 
 				draw_select_frame(true);
@@ -285,7 +338,7 @@ bool Board::on_button_press_event(GdkEventButton* ev)
 		else{
 			/** 之前已经选中棋子，现在是生成着法或取消*/
 			/** there has a selecter, now buider the moves or canel */
-			int dst_chessman = m_engine.get_piece(selected_x,selected_y);
+			int dst_chessman = m_engine.get_piece(selected_x,selected_y,is_rev_board);
 			if( (dst_chessman!=0) && ((selected_chessman &16)==(dst_chessman&16))){
 				/** 之前所选及现在选是同一色棋子, 变更棋子选择 */
 				/** change the select */
@@ -474,8 +527,8 @@ void Board::draw_chessman(int x, int y, int chessman)
 		return;
 
 	Gdk::Point p = get_coordinate(x, y);
-	int px = p.get_x() - 57 / 2;
-	int py = p.get_y() - 57 / 2;
+	int px = p.get_x() - chessman_width / 2;
+	int py = p.get_y() - chessman_width / 2;
 
 	ui_pixmap->draw_pixbuf(get_style()->get_black_gc(),chessman_images[chess_type],
 			0, 0, px, py, chessman_images[chess_type]->get_width(), chessman_images[chess_type]->get_height(), 
@@ -490,12 +543,12 @@ void Board::draw_select_frame(bool selected)
 	/** 目前要做的是根据棋子代号，获取它所在的棋盘9x10坐标*/
 	//Gdk::Point p = get_coordinate(selected_x, selected_y);
 	int sx,sy;
-	m_engine.get_xy_from_chess(selected_chessman,sx,sy);
+	m_engine.get_xy_from_chess(selected_chessman,sx,sy,is_rev_board);
 	Gdk::Point p = get_coordinate(sx, sy);
 
 
-	int px = p.get_x() - 57 / 2;
-	int py = p.get_y() - 57 / 2;
+	int px = p.get_x() - chessman_width / 2;
+	int py = p.get_y() - chessman_width / 2;
 
 
 	if (selected) {
@@ -537,7 +590,7 @@ void Board::draw_board()
 {
 	for (int i = 0; i < 9; i++) {
 		for (int j = 0; j < 10; j++) {
-			draw_chessman(i, j, m_engine.get_piece(i, j));
+			draw_chessman(i, j, m_engine.get_piece(i, j,is_rev_board));
 		}
 	}
 }
@@ -644,7 +697,7 @@ void Board::get_board_by_move(int f_step)
 int Board::try_move(int dst_x,int dst_y)
 {
 
-	int dst = m_engine.get_dst_xy(dst_x,dst_y);
+	int dst = m_engine.get_dst_xy(dst_x,dst_y,is_rev_board);
 	int src = m_engine.get_chessman_xy(selected_chessman);
 	int mv =  m_engine.get_move(src,dst);
 	int eat = m_engine.get_move_eat(mv);
@@ -698,10 +751,11 @@ int Board::try_move(int mv)
 			m_robot.send_ctrl_command("\n");
 			//user_player = 1-user_player;
 			if(!is_human_player()){
-				Glib::ustring str_cmd="go time "+to_msec_ustring(black_time)+" increment 0\n";
-				DLOG("send command to tell robot\n");
-				DLOG("%s\n",str_cmd.c_str());
-				m_robot.send_ctrl_command(str_cmd.c_str());
+				//Glib::ustring str_cmd="go time "+to_msec_ustring(black_time)+" increment 0\n";
+				//Glib::ustring str_cmd=Glib::ustring("go depth ")+search_depth+"\n";
+				char str_cmd[256];
+				sprintf(str_cmd,"go depth %d \n",m_search_depth);
+				m_robot.send_ctrl_command(str_cmd);
 				//m_robot.send_ctrl_command("go time 295000 increment 0\n");
 			}
 
@@ -709,17 +763,22 @@ int Board::try_move(int mv)
 			parent.change_play(is_human_player());
 			count_time=0;
 		}
+
 		/**被将死了*/
 		/** is it  mate */
-		if(m_engine.mate()){
-			parent.on_mate_game();
+		if(m_engine.mate() && is_human_player() ){
+			if(timer.connected())
+				timer.disconnect();
+			CSound::play(SND_CHECK);
+			parent.on_end_game(ROBOT_WIN);
 			DLOG("将军死棋\n");
+			return 0;
 		}
-
 		if(m_engine.get_checkby()){
 			CSound::play(SND_CHECK);
 			DLOG("将军===============\n");
 		}
+
 	}
 
 	return 0;
@@ -809,6 +868,12 @@ void Board::free_game()
 	m_engine.init_snapshot(start_fen);
 	redraw();
 }
+void Board::rev_game()
+{
+	is_rev_board=1-is_rev_board;
+	m_human_black=1-m_human_black;
+	redraw();
+}
 
 void Board::start_robot()
 {
@@ -819,6 +884,12 @@ void Board::start_robot()
 	new_game();
 }
 
+void Board::set_level_config(int _depth,int _idle,int _style,int _knowledge,int _pruning,int _randomness,bool _usebook)
+{
+	m_search_depth = _depth;
+	m_usebook = _usebook;
+
+}
 void Board::set_level()
 {
 
@@ -828,7 +899,10 @@ void Board::set_level()
 	m_robot.send_ctrl_command("setoption knowledge none\n");
 	m_robot.send_ctrl_command("setoption pruning  large\n");
 	m_robot.send_ctrl_command("setoption randomness large\n");
-	//m_robot.send_ctrl_command("ucci\n");
+	if(m_usebook)
+		m_robot.send_ctrl_command("setoption usebook false\n");
+	else
+		m_robot.send_ctrl_command("setoption usebook true\n");
 }
 
 void Board::new_game()
@@ -836,17 +910,28 @@ void Board::new_game()
 	m_status = FIGHT_STATUS;
 
 	m_engine.init_snapshot(start_fen);
-	set_level();
 	m_robot.send_ctrl_command("setoption newgame\n");
+	set_level();
 
 
 	moves_lines.clear();
 	moves_lines = postion_str + std::string(start_fen);
 	redraw();
 
-	parent.change_play(m_engine.red_player());
+	//parent.change_play(m_engine.red_player());
+	parent.change_play(is_human_player());
 
 	timer=Glib::signal_timeout().connect(sigc::mem_fun(*this,&Board::go_time),1000);
+	/**如果是用户选择黑方，则电脑先走棋 -- if user choose black,the robot go moves first*/
+	if(m_human_black){
+		moves_lines =moves_lines +std::string(" -- 0 1 ");
+		m_robot.send_ctrl_command(moves_lines.c_str());
+		m_robot.send_ctrl_command("\n");
+		char str_cmd[256];
+		sprintf(str_cmd,"go depth %d \n",m_search_depth);
+		m_robot.send_ctrl_command(str_cmd);
+
+	}
 
 }
 
@@ -872,13 +957,25 @@ bool Board::robot_log(const Glib::IOCondition& condition)
 		if(pos_ != std::string::npos){
 
 			printf("计算机同意和棋\n");
-
-			return true;
+			if (parent.on_end_game(ROBOT_DRAW)) {
+				if(timer.connected())
+					timer.disconnect();
+				return 1;
+			}
 		}
 		pos_=str_buf.find("resign");
 		if(pos_ != std::string::npos){
 
-			parent.on_mate_game();
+			if(timer.connected())
+				timer.disconnect();
+			parent.on_end_game(ROBOT_LOSE);
+			return true;
+		}
+		pos_=str_buf.find("nobestmove");
+		if(pos_ != std::string::npos){
+			if(timer.connected())
+				timer.disconnect();
+			parent.on_end_game(ROBOT_LOSE);
 			return true;
 		}
 		size_t pos=str_buf.find("bestmove");
@@ -925,3 +1022,20 @@ bool Board::go_time()
 
 	return true;
 }
+
+void Board::set_board_size(BOARDSIZE sizemode)
+{
+	switch(sizemode){
+		case BIG_BOARD:
+			is_small_board=false;
+			this->set_size_request(521,577);
+			break;
+		case SMALL_BOARD:
+			is_small_board=true;
+			this->set_size_request(221,277);
+			break;
+		default:
+			g_warn_if_reached();
+			break;
+	}
+}
