@@ -19,6 +19,10 @@
 #ifndef  CONFWINDOW_FILE_HEADER_INC
 #define  CONFWINDOW_FILE_HEADER_INC
 #include <gtkmm.h>
+#include "variablesmap.h"
+
+#define conf_ui DATA_DIR"/confwin.glade"
+
 class MainWindow;
 class ConfWindow: public Gtk::Window
 {
@@ -29,8 +33,19 @@ class ConfWindow: public Gtk::Window
 		void on_button_save();
 		void on_button_cancel();
 		bool on_delete_event(GdkEventAny*);
+		typedef sigc::signal<void> type_signal_quit;
+		type_signal_quit signal_quit()
+		{ return signal_quit_; }
+		void write_to_GMConf();
 	private:
+		typedef Glib::RefPtr < Gtk::Builder> GBuilderXML;
+		GBuilderXML			vbox_xml;
 		MainWindow* parent;
+		type_signal_quit signal_quit_;
+		VariablesMap* m_pVariablesMap;
+		bool				m_size_big;
+		bool				m_usebook;
+		Glib::ustring			m_depth;
 
 };
 
