@@ -32,8 +32,10 @@ void Engine::clean_board()
 {
 	int i,j;
 	for(i=0;i<16;i++)
-		for(j=0;j<16;j++)
+		for(j=0;j<16;j++){
 			chessboard[i*16+j] =0;
+			revchessboard[i*16+j] =0;
+			}
 
 	for(i=0;i<3;i++)
 		for(j=0;j<16;j++)
@@ -208,6 +210,7 @@ void Engine::init_snapshot(const char* fen)
 	fen_snapshots.push_back(std::string(fen));
 	move_snapshots.push_back(0);
 
+	sync_board();
 	/*
 	int i,j;
 	for(i=0;i<16;i++)
@@ -226,24 +229,45 @@ void Engine::get_snapshot(int num)
 	clean_board();
 	from_fens(fens.c_str());
 
+	sync_board();
+
 }
-int Engine::get_piece(int rx,int ry)
+int Engine::get_piece(int rx,int ry,bool rev)
 {
 	int site=0;
 	site = get_coord(rx + 3,ry + 3);
-	return chessboard[site];
+	if(rev)
+		return revchessboard[site];
+	else
+		return chessboard[site];
 
 }
-int Engine::get_dst_xy(int rx, int ry)
+
+int Engine::get_rev_piece(int rx,int ry)
 {
-	return get_coord(rx+3,ry+3);
+	int site=0;
+	site = get_coord(rx + 3,ry + 3);
+	return revchessboard[site];
+
+}
+int Engine::get_dst_xy(int rx, int ry,bool rev)
+{
+	if(rev)
+		return get_coord(11-rx,12-ry);
+	else
+		return get_coord(rx+3,ry+3);
 }
 
-void Engine::get_xy_from_chess(int f_chess,int& x,int& y)
+void Engine::get_xy_from_chess(int f_chess,int& x,int& y,bool rev)
 {
 	int xy=get_chessman_xy(f_chess);
-	x=RANK_X(xy)-3;
-	y=RANK_Y(xy)-3;
+	if(rev){
+		x=11-RANK_X(xy);
+		y=12-RANK_Y(xy);
+	}else{
+		x=RANK_X(xy)-3;
+		y=RANK_Y(xy)-3;
+	}
 }
 
 char Engine::get_iccs_y(int nArg)
@@ -628,7 +652,7 @@ bool Engine::logic_move(int mv)
 	/** 判断是否过河的方法，dst & 0x80,在下方是非0, 上方是0 */
 	/** 获取要移动棋子的类型*/
 	int chess_t = get_chessman_type(chessboard[src]);
-	DLOG("逻辑判断棋子chesboard[src] = %d  %d\n",chessboard[src],chess_t);
+	DLOG("逻辑判断棋子chessboard[src] = %d  %d\n",chessboard[src],chess_t);
 	switch(chess_t){
 		/** 将/帅的着法，同一纵线或横线，移动只一个单位，在九宫内*/
 		case RED_KING:
@@ -881,7 +905,7 @@ int Engine::do_move(int mv)
 	uint32_t hanzi = iccs_to_hanzi(iccs);
 	Glib::ustring mv_line = hanzi_to_move_chinese(hanzi);
 
-	DLOG(" src = %x dst = %x mv = %d\n chessboard[src]= %d , chessboard[dst] = %d,eated = %d\n",src,dst,mv,chessboard[src],chessboard[dst],eated);
+	DLOG(" src = %x dst = %x mv = %d\n chessboard[src]= %d , chessboard[dst] = %d,taken = %d\n",src,dst,mv,chessboard[src],chessboard[dst],eated);
 
 	//if(eated != chessboard[dst])
 	//	return -1;
@@ -890,7 +914,7 @@ int Engine::do_move(int mv)
 	 * 则要将这个棋子的位置值置0,表示被吃掉，不再出现在棋盘上
 	 */
 	if(eated !=0){
-		DLOG(" %d has been eat\n",chessboard[dst]);
+		DLOG(" %d taken\n",chessboard[dst]);
 		chessmans[eated]=0;
 	}
 	chessboard[dst] = chessboard[src];
@@ -909,6 +933,8 @@ int Engine::do_move(int mv)
 
 	add_move_chinese(mv_line);
 
+	/** 同步反转棋盘*/
+	sync_board();
 	return 0;
 }
 
@@ -931,6 +957,9 @@ void Engine::undo_move(int mv)
 	move_snapshots.pop_back();
 	fen_snapshots.pop_back();
 	move_chinese.pop_back();
+
+	/** 同步反转棋盘*/
+	sync_board();
 }
 
 int Engine::iccs_str_to_move(const std::string& iccs_str)
@@ -1622,13 +1651,17 @@ uint32_t Engine::hanzi_to_iccs(uint32_t f_hanzi)
 	/** cman_type 是棋子类型*/
 	int cman_type = fen_to_piece(c_hanzi.word[0]);
 	int src_x,src_y,dst_x,dst_y;
-	int num;
+	int num =0;
 
 	DLOG("who is going black=%d\n",black_player);
+	num += 16*black_player;
+	/*
 	if(black_player)
 		num = 16;
 	else
 		num= 0;
+	*/
+	g_log("Engine",G_LOG_LEVEL_INFO,"num = %d\n",num);
 
 	if(0 == cman_type ){
 		/** 处理帅将的源地址*/
@@ -2145,3 +2178,12 @@ error_out:
 	printf(" %s error\n",__FUNCTION__);
 	return 0;
 }
+
+void Engine::sync_board()
+{
+	int i;
+	for(i=0;i<255;i++)
+		revchessboard[i]=chessboard[254-i];
+	revchessboard[255]=0;
+}
+
