@@ -21,10 +21,13 @@
 #include <gtkmm/treeselection.h>
 #include <gtkmm/aboutdialog.h>
 #include <fstream>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include "BookView.h"
 #include "ConfWindow.h"
+#include "gmchess.h"
 
-#define version "0.20.2"
+#define version "0.20.4"
 
 Glib::ustring ui_info =
 "<ui>"
@@ -36,6 +39,7 @@ Glib::ustring ui_info =
 "			<menuitem action='FileQuit'/>"
 "        	</menu>"
 "		<menu action='ViewMenu'>"
+"			<menuitem action='RevPlay'/>"
 "			<menuitem action='ViewPreferences'/>"
 "		</menu>"
 "		<menu action='GameMenu'>"
@@ -114,6 +118,8 @@ MainWindow::MainWindow():menubar(NULL)
 	btn_rue->signal_clicked().connect(
 			sigc::mem_fun(*this, &MainWindow::on_rue_game));
 
+	//init_conf();
+
 	board= Gtk::manage(new Board(*this));
 	box_board->pack_start(*board);
 
@@ -145,7 +151,7 @@ MainWindow::MainWindow():menubar(NULL)
 	m_treeview.signal_button_press_event().connect(sigc::mem_fun(*this,
 				&MainWindow::on_treeview_click),false);
 
-	change_status();
+	set_status();
 
 	//Gtk::ScrolledWindow* scroll_book=dynamic_cast<Gtk::ScrolledWindow*>
 	//	(ui_xml->get_widget("scrolledwin_book"));
@@ -169,7 +175,9 @@ MainWindow::MainWindow():menubar(NULL)
 	p2_step_time= dynamic_cast<Gtk::Label*>(ui_xml->get_widget("P2_step_time"));
 #endif
 
-	init();
+	init_conf();
+	this->signal_check_resize().connect(
+			sigc::mem_fun(*this,&MainWindow::on_size_change));
 	
 	/** test for rgba */
 	Glib::RefPtr<const Gdk::Colormap> colormap_ = this->get_screen()->get_rgba_colormap();
@@ -179,6 +187,14 @@ MainWindow::MainWindow():menubar(NULL)
 	p1_image->hide();
 	p2_image->hide();
 
+	//set_default_size(851,727);
+	if(atoi(GMConf["desktop_size"].c_str()) == 1)
+		board->set_board_size(BIG_BOARD);
+	else
+		board->set_board_size(SMALL_BOARD);
+	int _depth = atoi(GMConf["engine_depth"].c_str());
+	g_log("Mainwindow",G_LOG_LEVEL_INFO,"depth %d",_depth);
+	board->set_level_config(_depth,0,0,0,0,0,atoi(GMConf["usebook"].c_str()));
 }
 
 MainWindow::~MainWindow()
@@ -186,15 +202,14 @@ MainWindow::~MainWindow()
 	this->pop_colormap();
 }
 
+void MainWindow::on_size_change()
+{
+	int _wid = board->get_width();
+	board->configure_board(_wid);
+}
+
 void MainWindow::change_play(bool player)
 {
-	/*
-	if( FIGHT_STATUS != board->get_status()){
-		p1_image->remove(*(board->get_p1_image()));
-		p2_image->remove(*(board->get_p2_image()));
-		return ;
-	}
-	*/
 	if(player){
 		p1_image->hide();
 		p2_image->show();
@@ -205,15 +220,81 @@ void MainWindow::change_play(bool player)
 	}
 }
 
-
-void MainWindow::init()
+void MainWindow::save_conf()
 {
+
 	char buf[512];
-	char* homedir = getenv("HOME");
-	snprintf(buf, 512,"%s/.gmchess/book",homedir);
-	m_bookview->load_book_dir(buf);
+	std::string homedir=Glib::get_user_config_dir();
+	snprintf(buf, 512,"%s/gmchess/config",homedir.c_str());
+	std::ofstream file(buf);
+	std::string line;
+	std::map<std::string,std::string>::iterator iter=GMConf.begin();
+	for(;iter != GMConf.end(); ++iter)
+	{
+		line = iter->first + "\t=\t" + iter->second;
+		file << line << std::endl;
+	}
+	file.close();
 
 }
+
+void MainWindow::init_conf()
+{
+	char buf[512];
+	char book_dir[512];
+	std::string homedir=Glib::get_user_config_dir();
+	snprintf(book_dir, 512,"%s/gmchess/book",homedir.c_str());
+	m_bookview->load_book_dir(book_dir);
+	snprintf(buf,512,"%s/gmchess/config",homedir.c_str());
+	std::ifstream file(buf);
+	if(!file){
+		char homepath[512];
+		snprintf(homepath,512,"%s/gmchess/",homedir.c_str());
+		mkdir(homepath,S_IRUSR|S_IWUSR|S_IXUSR);
+		GMConf["usebook"]="1";
+		GMConf["desktop_size"] = "1"; //0--small,1--big
+		GMConf["engine_depth"] ="5";
+		save_conf();
+		return;
+	}
+	
+	std::string line;
+	std::string name;
+	std::string key;
+
+	if(file){
+		while(std::getline(file,line)){
+			size_t pos= line.find_first_of("=");
+			if(pos==std::string::npos)
+				continue;
+			name = line.substr(0,pos);
+			key = line.substr(pos+1,std::string::npos);
+			// 下面这2个把所有的空格都去掉了
+			//key.erase(std::remove_if(key.begin(), key.end(), IsBlank()), key.end());
+			//name.erase(std::remove_if(name.begin(), name.end(), IsBlank()), name.end());
+			size_t pos1 = 0;
+			size_t pos2 = 0;
+			size_t len = 0;
+			pos1 = name.find_first_not_of(" \t");
+			pos2 = name.find_last_not_of(" \t");
+			if (pos1 == std::string::npos || pos2 == std::string::npos)
+				continue;
+			len = pos2 - pos1 + 1;
+			name = name.substr(pos1, len);
+
+			pos1 = key.find_first_not_of(" \t");
+			pos2 = key.find_last_not_of(" \t");
+			if (pos1 == std::string::npos || pos2 == std::string::npos)
+				continue;
+			len = pos2 - pos1 + 1;
+			key = key.substr(pos1, len);
+
+			GMConf.insert(std::pair<std::string,std::string>(name,key));
+		}
+	}
+	file.close();
+}
+
 void MainWindow::set_comment(const std::string& f_comment)
 {
 	text_comment->set_wrap_mode(Gtk::WRAP_WORD);
@@ -302,6 +383,8 @@ void MainWindow::init_ui_manager()
 			sigc::mem_fun(*this, &MainWindow::on_menu_war_to_ai));
 	action_group->add(Gtk::Action::create("FreePlay",Gtk::Stock::MEDIA_PLAY,_("Free Play")),
 			sigc::mem_fun(*this, &MainWindow::on_menu_free_play));
+	action_group->add(Gtk::Action::create("RevPlay",Gtk::Stock::MEDIA_PLAY,_("Change Side")),
+			sigc::mem_fun(*this, &MainWindow::on_menu_rev_play));
 	//Help menu:
 	action_group->add(Gtk::Action::create("HelpMenu", _("_Help")));
 	action_group->add(Gtk::Action::create("Help", Gtk::Stock::HELP),
@@ -490,7 +573,12 @@ void MainWindow::on_menu_free_play()
 {
 	board->free_game();
 	m_refTreeModel->clear();
-	change_status();
+	set_status();
+}
+void MainWindow::on_menu_rev_play()
+{
+	board->rev_game();
+
 }
 
 void MainWindow::on_menu_file_quit()
@@ -503,11 +591,21 @@ void MainWindow::on_menu_view_preferences()
 {
 	if(NULL == confwindow){
 		confwindow = new ConfWindow(this);
+		confwindow->signal_quit().connect(sigc::mem_fun(*this, &MainWindow::on_conf_window_quit));
 		confwindow->raise();
 	}else
 		confwindow->raise();
 }
 
+void MainWindow::on_conf_window_quit()
+{
+	save_conf();
+
+	if(atoi(GMConf["desktop_size"].c_str()) == 1)
+		board->set_board_size(BIG_BOARD);
+	else
+		board->set_board_size(SMALL_BOARD);
+}
 void MainWindow::on_conf_window_close()
 {
 	if(NULL !=confwindow){
@@ -638,12 +736,12 @@ void MainWindow::set_information()
 	text = text + _("Variation:  ")+board_info.variation+"\n";
 	info->set_label(text);
 	info->set_ellipsize(Pango::ELLIPSIZE_END);
-	change_status();
+	set_status();
 
 }
 
 
-void MainWindow::change_status()
+void MainWindow::set_status()
 {
 	int f_status = board->get_status();
 	bool f_use=1;
@@ -734,7 +832,7 @@ void MainWindow::on_begin_game()
 	}
 	m_refTreeModel->clear();
 	board->start_robot();
-	change_status();
+	set_status();
 	btn_begin->set_sensitive(false);
 }
 
@@ -752,7 +850,7 @@ void MainWindow::on_lose_game()
             case (Gtk::RESPONSE_OK): {
                 m_refTreeModel->clear();
                 board->free_game();
-                change_status();
+                set_status();
                             break;
                     }
 
@@ -796,28 +894,45 @@ void MainWindow::on_rue_game()
 	}
 
 }
-void MainWindow::on_mate_game()
+bool MainWindow::on_end_game(OVERSTATUS _over)
 {
+	Glib::ustring msg ;
+	switch(_over){
+		case ROBOT_WIN:
+			//robot win
+			msg=_("You are Lose! \n Are you want to start a new game?");
+			break;
+		case ROBOT_LOSE:
+			//robot lose
+			msg=_("Congratuations ! YOU WIN!\n start a new game click OK");
+			break;
+		case ROBOT_DRAW:
+			//robot want to draw
+			msg=_("Computer want to draw with you, are you agree?");
+			break;
+		default:
+			break;
+
+	}
 	Gtk::MessageDialog dialog(*this, _("Game End"), false,
                                   Gtk::MESSAGE_QUESTION,
                                   Gtk::BUTTONS_OK_CANCEL);
-		Glib::ustring msg =_("Mate!\n Are you want to start a new game?");
+		//Glib::ustring msg =_("Mate!\n Are you want to start a new game?");
 		dialog.set_secondary_text(msg);
 		int result = dialog.run();
 		switch (result) {
 			case (Gtk::RESPONSE_OK): {
 				m_refTreeModel->clear();
 				board->new_game();
-                	        break;
+				return true;
                 	}
 
 			case (Gtk::RESPONSE_CANCEL): {
-				return;
-                	        break;
+				return false;
                 	}
 
 			default: {
-				return;
+				return false;
                 	        break;
                 	}
 		}
